# routines common to the auxiliary programs                       -*- perl -*-
# $Id: psl-aux.pl,v 1.3 1996/02/23 00:21:38 olaf Exp $

# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# (at your option) any later version.
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# ----------------------------------------------------------------------------

require "lockfiles.pl";
require "psl-parm.pl";

sub auxinit
{
    local($name) = @_;
    chdir("$newslib") ||
       die "chdir($newslib) failed";
    $SIG{"HUP"} = "IGNORE";
    $SIG{"INT"} = "signal_catch";
    $SIG{"TERM"} = "signal_catch";
    $SIG{"PIPE"} = "signal_catch";
    while (1) {
       if (&lockfile("LOCK", "")) {
           last;
       }
       if ($?<0) {
           die "Can't lock the news system";
       } else {
           print "News system locked, waiting...\n";
           sleep 10;
       }
    }
    open(LOG, ">>$globlog") || die "open log";
    select(LOG); $|=1;
    $d=`date`; print LOG "## $name << $d";
    select(STDOUT); $|=1;

    print STDOUT "<< ";
    &Read("active", active);
    &Read("active.times", activetimes);
    &Read("newsgroups", newsgroups);
    print STDOUT "\n";
    $tstr = sprintf("%d %s@%s (%s)", time, $logname, $hostname, $name);
}

sub auxexit
{
    local($dowrite) = @_;
    print STDOUT "\n";
    $d=`date`; print LOG "## >> $d\n";
    $SIG{"INT"} = "IGNORE";
    $SIG{"TERM"} = "IGNORE";
    $SIG{"PIPE"} = "IGNORE";
    if ($dowrite) {
       print STDOUT ">> ";
       &Write("active", active, " ");
       &Write("active.times", activetimes, " ");
       &Write("newsgroups", newsgroups, "\t");
       print STDOUT "\n";
       foreach $i ("active", "newsgroups") {
           unlink("${i}.o");
           rename("$i", "${i}.o") && rename("${i}+", "$i");
       }
       unlink("active.times.o");
       rename("active.times", "active.times.o") &&
           system("sort -n +1 active.times+ > active.times") ||
               unlink("active.times+");
    }
    unlink("LOCK");
    exit 0;
}

sub addgroup
{
    local($g,$f) = @_;
    print LOG "created $g $f\n";
    $active{$g} = "0000000000 00001 $f";
    $activetimes{$g} = $tstr;
    $newsgroups{$g} = ($f eq "m") ? "(Moderated)" : "";
}

sub delgroup
{
    local($g) = @_;
    print LOG "deleted $g\n";
    $deleted{$g} = 1;
    if (open(TBD, ">>$newslib/dirs.tbd")) {
       $g =~ tr:.:/:;
       print TBD "$g\n";
       close TBD;
    }
}


# ----------------------------------------------------------------------------

sub Read
{
    local($fn,*ar) = @_;
    open(F, "<$fn") || die "open $fn";
    print STDOUT "$fn ";
    while(<F>) {
       chop;
       ($g,$x) = split(/[ \t]+/, $_, 2);
       $ar{$g} = $x;
    }
    close F;
}

sub Write
{
    local($fn,*ar,$s) = @_;
    open(F, ">$fn+") || die "open $fn+";
    print STDOUT "$fn ";
    foreach $i (sort keys %ar) {
       if (!$deleted{$i}) {
           printf(F "%s%s%s\n", $i, $s, $ar{$i});
       }
    }
    close F;
}

sub signal_catch
{
    print LOG "\n## break\n";
    unlink("LOCK");
    exit 1;
}

sub die_catch
{
    print STDERR "\n$_[0]";
    print LOG "\n## $_[0]";
    unlink("LOCK");
    exit 1;
}

# ----------------------------------------------------------------------------

sub open_batch
{
    if ($dfcommand && !$opt_s) {
       open(DF, sprintf("$dfcommand|", $inputdir))
           || &die_catch("df");
       $_ = <DF>;
       close DF;
       split;
       if (1024*$_[$dffield] < 2*$opt_b) {
           &die_catch("Filesystem for $inputdir full");
       }
    }
    while (1) {
       ++$batchcount;
       $batchname = sprintf("$inputdir/mvj.%05d%04d", $$, $batchcount);
       last if (!-e $batchname);
    }
    open (BATCH, ">$batchname")
       || &die_catch("open $batchname");
    select BATCH; $|=1;
    $batchsize = 0;
}

sub close_batch
{
    close BATCH;
    if ($batchsize) {
       ++$Sbatch;
       while (1) {
           rename($batchname, sprintf("$inputdir/%d.t", time)) && last;
           sleep 2;
       }
    } else {
       unlink $batchname;
    }
}

# ----------------------------------------------------------------------------
1;
